iilabs3d_bench_seq = [
    "nav_a_diff",
    "nav_a_omni",
    "loop",
    "slippage",
    "ramp",
    "elevator",
]

iilabs3d_calib_seq_imu_intrinsic = [
    "calib_xsens_imu",
    "calib_livox_imu",
    "calib_ouster_imu",
]

iilabs3d_calib_seq_wheel_odom = [
    "calib_square_ccw_no_rot",
    "calib_square_ccw_rot",
    "calib_square_cw_no_rot",
    "calib_square_cw_rot",
]

iilabs3d_calib_seq_extrinsic = [
    "calib_full_excite",
    "calib_infinity",
]

iilabs3d_lidar_sensors = [
    "livox_mid_360",
    "ouster_os1_64",
    "robosense_rs_helios_5515",
    "velodyne_vlp_16",
]